/*
 * File:     ContinuousRateTSTrack.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.client.timeseries;


/**
 * Implementation of a time series data track.
 * Data are stored in a flat list or array of values; in combination with the 
 * (fixed) sample rate it is possible to find a time-value pair. 
 */
public class ContinuousRateTSTrack extends AbstractTSTrack {
    private float sampleRate;
    private float msPerSample;
    private float[] data;

    /**
 * Constructor.
 */
    public ContinuousRateTSTrack() {
        super();
    }

    /**
 * @see mpi.eudico.client.timeseries.TimeSeriesTrack#setSampleRate(int)
 */
    public void setSampleRate(float rate) {
        sampleRate = rate;
        msPerSample = 1000 / sampleRate;
    }

    /**
 * @see mpi.eudico.client.timeseries.TimeSeriesTrack#getSampleRate()
 */
    public float getSampleRate() {
        return sampleRate;
    }

    /**
 * @see mpi.eudico.client.timeseries.TimeSeriesTrack#getSampleCount()
 */
    public int getSampleCount() {
        if (data == null) {
            return 0;
        }

        return data.length;
    }

    /**
 * Returns an array of floats.
 * 
 * @see mpi.eudico.client.timeseries.TimeSeriesTrack#getData()
 */
    public Object getData() {
        return data;
    }

    /**
 * Sets the data of this tracks.
 * Currently this method only accepts an array of floats; in any other case 
 * an IllegalArgumentException will be thrown.
 * 
 * @see mpi.eudico.client.timeseries.TimeSeriesTrack#setData(java.lang.Object)
 * @throws IllegalArgumentException when the data is provided in anything else
 * but an array of floats
 */
    public void setData(Object data) {
        if (!(data instanceof float[])) {
            throw new IllegalArgumentException(
                "This track only accepts an array of floats");
        }

        this.data = (float[]) data;
    }

    /**
 * Returns a calculated index based on the time, time offset and the sample rate.
 * There is no garuantee that the index is within the data array's bounds.
 * 
 * @see mpi.eudico.client.timeseries.AbstractTSTrack#getIndexForTime(long)
 */
    public int getIndexForTime(long time) {
        int index = -1;

        if ((time + timeOffset) >= 0) {
            index = (int) ((time + timeOffset) / msPerSample);
        }

        return index;
    }

    /**
 * @see mpi.eudico.client.timeseries.AbstractTSTrack#getTimeForIndex(int)
 */
    public long getTimeForIndex(int index) {
        if (index < 0) {
            throw new ArrayIndexOutOfBoundsException(
                "Index should be greater than or equal to 0");
        }

        long time = 0L;

        if (data != null) {
            if (index >= data.length) {
                throw new ArrayIndexOutOfBoundsException("Index (" + index +
                    ") is greater than " + (data.length - 1));
            }

            time = (long) (index * msPerSample);
        }

        return time + timeOffset;
    }
}
