/*
 * File:     TimeSeriesViewer.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.client.timeseries;

import mpi.eudico.client.annotator.Constants;

import mpi.eudico.client.annotator.viewer.TimeRuler;
import mpi.eudico.client.annotator.viewer.TimeScaleBasedViewer;

import mpi.eudico.client.mediacontrol.ControllerEvent;

import mpi.eudico.client.util.TimeFormatter;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JScrollBar;
import javax.swing.SwingConstants;
import javax.swing.UIManager;


/**
 * A viewer that can handle one or more TimeSeriesTrack panels, 
 * adding a horizontal ruler, a crosshair, selection (time interval) etc. 
 */
public class TimeSeriesViewer extends TimeScaleBasedViewer
    implements ComponentListener, AdjustmentListener {
    private BufferedImage bi;
    private Graphics2D big2d;
    private int imageWidth;
    private int imageHeight;
    private Font font;
    private FontMetrics metrics;
    private int rulerHeight;
    private int vertRulerWidth;
    private TimeRuler ruler;
    private int verticalScrollOffset;
    private AffineTransform identity;
    private long crossHairTime;
    private int crossHairPos;
    private long intervalBeginTime;
    private long intervalEndTime;
    private float msPerPixel;

    // vertical scrolling
    private JScrollBar scrollBar;

    /** default scrollbar width */
    private final int defBarWidth;

    /** a list of trackpanels */
    private List trackPanels;
    private int trackPanelHeight;
    private Insets panelMargins;

    /**
     * Creates a new TimeSeriesViewer instance
     */
    public TimeSeriesViewer() {
        initViewer();
        defBarWidth = getDefaultBarWidth();

        addComponentListener(this);
        setOpaque(true);

        paintBuffer();
    }

    /**
 * Performs the initialization of fields and sets up the viewer.<br>
 */
    private void initViewer() {
        trackPanels = new ArrayList(4);
        font = Constants.DEFAULTFONT;
        setFont(font);
        metrics = getFontMetrics(font);
        ruler = new TimeRuler(font, TimeFormatter.toString(0));
        rulerHeight = ruler.getHeight();
        vertRulerWidth = 43;
        msPerPixel = 10f;
        crossHairTime = 0L;
        crossHairPos = 0;

        //dragStartTime = 0;
        imageWidth = 0;
        imageHeight = 0;
        trackPanelHeight = 300;
        panelMargins = new Insets(3, 3, 3, 0);
        identity = new AffineTransform();

        verticalScrollOffset = 0;
        scrollBar = new JScrollBar(JScrollBar.VERTICAL, 0, 50, 0, 200);
        scrollBar.setBlockIncrement(trackPanelHeight);

        //scrollBar.setUnitIncrement(5);
        scrollBar.addAdjustmentListener(this);
        setLayout(null);
        add(scrollBar);
    }

    /**
 * Retrieves the default, platform specific width of a scrollbar.
 *
 * @return the default width, or 20 when not found
 */
    private int getDefaultBarWidth() {
        int width = 20;

        if (UIManager.getDefaults().get("ScrollBar.width") != null) {
            width = ((Integer) (UIManager.getDefaults().get("ScrollBar.width"))).intValue();
        }

        return width;
    }

    /**
 * Override <code>JComponent</code>'s paintComponent to paint:<br>
 * - a BufferedImage with a ruler and the tags<br>
 * - the cursor / crosshair
 *
 * @param g the graphics object
 */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2d = (Graphics2D) g;
        int h = getHeight();

        if (bi != null) {
            g2d.translate(0, -verticalScrollOffset);
            g2d.drawImage(bi, 0, 0, this);
            g2d.translate(0, verticalScrollOffset);
        }

        /*
if ((crossHairPos >= 0) && (crossHairPos <= imageWidth)) {
    // prevents drawing outside the component on Mac
    g2d.setColor(Constants.CROSSHAIRCOLOR);
    g2d.drawLine(crossHairPos, 0, crossHairPos, h);
}
*/
    }

    /**
 * Paint to a buffer.<br>
 * First paint the top ruler, next the annotations of the current tier.
 */
    private void paintBuffer() {
        if ((getWidth() <= 0) || (getHeight() <= 0)) {
            return;
        }

        if (imageWidth != (getWidth() - defBarWidth)) {
            imageWidth = getWidth() - defBarWidth;
        }

        int h = (trackPanels.size() * trackPanelHeight) + rulerHeight;

        if (imageHeight != h) {
            imageHeight = (getHeight() > h) ? getHeight() : h;
        }

        intervalEndTime = intervalBeginTime + (int) (imageWidth * msPerPixel);

        if ((bi == null) || (bi.getWidth() < imageWidth) ||
                (bi.getHeight() < imageHeight)) {
            bi = new BufferedImage(imageWidth, imageHeight,
                    BufferedImage.TYPE_INT_RGB);
            big2d = bi.createGraphics();
        }

        big2d.setColor(Color.WHITE);
        big2d.fillRect(0, 0, imageWidth, bi.getHeight());

        TSTrackPanelImpl panel;
        int y = 0;

        for (int i = 0; i < trackPanels.size(); i++) {
            panel = (TSTrackPanelImpl) trackPanels.get(i);
            big2d.translate(0, y);
            panel.paint(big2d);
            y += panel.getHeight();
        }

        big2d.setTransform(identity); //reset transform

        /*paint time ruler */
        big2d.setColor(Constants.DEFAULTFOREGROUNDCOLOR);
        big2d.translate(vertRulerWidth, y);
        ruler.paint(big2d, intervalBeginTime, imageWidth, (int) msPerPixel,
            SwingConstants.TOP);

        big2d.setTransform(identity); //reset transform
        repaint();
    }

    /**
 * Adds a trackpanel to the viewer.
 * The panel's settings are forced to comply to the viewer's settings.
 * 
 * @param trackPanel the trackpanel to add
 */
    public void addTSTrackPanel(TSTrackPanelImpl trackPanel) {
        trackPanel.setHeight(trackPanelHeight);
        trackPanel.setWidth(getWidth() - defBarWidth);
        trackPanel.setMargin(panelMargins);
        trackPanel.setRulerWidth(vertRulerWidth - panelMargins.left);
        trackPanel.setMsPerPixel(msPerPixel);
        trackPanels.add(trackPanel);
        paintBuffer();
    }

    /**
 * Removes a trackpanel from the viewer
 *
 * @param trackPanel the trackpanel to remove
 */
    public void removeTSTrackPanel(TSTrackPanelImpl trackPanel) {
        trackPanels.remove(trackPanel);
    }

    /**
 * Returns the width for the vertical ruler.
 *
 * @return the width for the vertical ruler
 */
    public int getVerticalRulerWidth() {
        return vertRulerWidth;
    }

    /**
 * Sets the width for the vertical ruler. 
 *
 * @param rulerWidth the width for the vertical ruler
 */
    public void setVerticalRulerWidth(int rulerWidth) {
        vertRulerWidth = rulerWidth;
    }

    /**
 * Returns the vertical scroll offset.
 *
 * @return the vertical scroll offset
 */
    public int getVerticalScrollOffset() {
        return verticalScrollOffset;
    }

    /**
 * Sets the vertical scroll offset on this component. 
 *
 * @param offset the new vertical scroll offset
 */
    public void setVerticalScrollOffset(int offset) {
        verticalScrollOffset = offset;
        repaint();
    }

    /**
 * Update the values of the scrollbar.<br>
 * Called after a change in the number of visible tracks.
 */
    private void updateScrollBar() {
        int value = scrollBar.getValue();
        int max = (imageHeight > getHeight()) ? imageHeight : getHeight();

        // before changing scrollbar values do a setValue(0), otherwise
        // setMaxiimum and/or setVisibleAmount will not be accurate
        scrollBar.setValue(0);
        scrollBar.setMaximum(max);
        scrollBar.setVisibleAmount(getHeight());

        if ((value + getHeight()) > max) {
            value = max - getHeight();
        }

        scrollBar.setValue(value);
        scrollBar.revalidate();
    }

    /**
 * @see mpi.eudico.client.annotator.viewer.TimeScaleBasedViewer#updateTimeScale()
 */
    public void updateTimeScale() {
    }

    /**
 * @see mpi.eudico.client.annotator.viewer.AbstractViewer#controllerUpdate(mpi.eudico.client.mediacontrol.ControllerEvent)
 */
    public void controllerUpdate(ControllerEvent event) {
    }

    /**
 * @see mpi.eudico.client.annotator.viewer.AbstractViewer#updateSelection()
 */
    public void updateSelection() {
    }

    /**
 * @see mpi.eudico.client.annotator.viewer.AbstractViewer#updateActiveAnnotation()
 */
    public void updateActiveAnnotation() {
        // ignore
    }

    /**
 * @see mpi.eudico.client.annotator.viewer.AbstractViewer#updateLocale()
 */
    public void updateLocale() {
    }

    /**
 * Stub.
 *
 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
 */
    public void componentHidden(ComponentEvent e) {
    }

    /**
 * Stub.
 *
 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
 */
    public void componentMoved(ComponentEvent e) {
    }

    /**
 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
 */
    public void componentResized(ComponentEvent e) {
        TSTrackPanelImpl panel;

        for (int i = 0; i < trackPanels.size(); i++) {
            panel = (TSTrackPanelImpl) trackPanels.get(i);
            panel.setWidth(getWidth() - defBarWidth);
        }

        paintBuffer();

        scrollBar.setBounds(getWidth() - defBarWidth, 0, defBarWidth,
            getHeight());
        scrollBar.revalidate();
        updateScrollBar();
    }

    /**
 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
 */
    public void componentShown(ComponentEvent e) {
        paintBuffer();

        scrollBar.setBounds(getWidth() - defBarWidth, 0, defBarWidth,
            getHeight());
        scrollBar.revalidate();
        updateScrollBar();
    }

    /**
 * Handle scrolling of the viewer image.
 *
 * @param e DOCUMENT ME!
 */
    public void adjustmentValueChanged(AdjustmentEvent e) {
        int value = e.getValue();

        setVerticalScrollOffset(value);
    }
}
